-- main.lua: contains main game loop, and practically everything else
-- last modified on Mar 09, 2012

function love.load()
	math.randomseed(os.time())
	smooth = true
	pause = false
	length = 16
	segments = 4
	scale = 1
	scalespeed = 2
	angle = 0
	width = love.graphics.getWidth()
	height = love.graphics.getHeight()
	hyp = math.sqrt(width^2 + height^2)
	stars = {}
	for i=1, 48 do
		create(hyp)
	end
	texture = makeTexture()
end

function love.update(dt)
	if not pause then
		for i=#stars, 1, -1 do
			stars[i].x = stars[i].x + (stars[i].z * dt * scale * 32)	-- "feels right"
			if stars[i].x > width + (stars[i].z * length) then
				table.remove(stars, i)
				create(1)
			end
		end
	end
	if love.keyboard.isDown("left") then
		angle = angle - dt
	elseif love.keyboard.isDown("right") then
		angle = angle + dt
	elseif love.keyboard.isDown("up") then
		scale = scale + (dt * scalespeed)
	elseif love.keyboard.isDown("down") then
		scale = scale - (dt * scalespeed)
	end
end

function love.draw()
	if angle ~= 0 then
		love.graphics.translate(width/2, height/2)
		love.graphics.rotate(angle)
		love.graphics.translate(-width/2, -height/2)
	end
	if smooth then
		for i=1, #stars do
			local sz = stars[i].z / length * scale
			love.graphics.draw(texture, stars[i].x, stars[i].y - 1, 0, -sz * scale, math.max(1, sz))
		end
	else 
		for i=1, #stars do
			local size = math.ceil(stars[i].z / length)
			local ext_z = stars[i].z * scale	
			for s=segments, 1, -1 do		-- count down so that we end at 255 opacity
				love.graphics.setColor(255,255,255, 255 - (64 * (s - 1)))
				love.graphics.rectangle("fill", stars[i].x - (ext_z * s), stars[i].y, ext_z, size)
			end
			love.graphics.circle("fill", stars[i].x, stars[i].y + (size / 2), (size / 2))
		end
	end
end

function love.keypressed(key)
	if key == "escape" then
		love.event.push("q")
	elseif key == "-" or key == "_" or key == "kp-" then
		table.remove(stars, 1)
	elseif key == "=" or key == "+" or key == "kp+" then
		create(1)
	elseif key == "tab" then
		smooth = not smooth
	elseif key == " " then
		pause = not pause
	end
end

function create(spread)
	local lz = math.random()^2 * (length * segments) + 1	-- creates a non-uniform distribution of z positions
	local lx = math.random(0, spread) - lz			-- x position
	local ly = math.random(0, hyp) - (hyp * .25)	-- y position
	stars[#stars + 1] = {x = lx, y = ly, z = lz}
end

function makeTexture()
	local data = love.image.newImageData(64,4)
	for i=0, 63 do	-- there is a pixel of padding top/bottom in the texture
		data:setPixel(i, 1, 255, 255, 255, 255 - (i * 4))
		data:setPixel(i, 2, 255, 255, 255, 255 - (i * 4))
	end
	return love.graphics.newImage(data)
end
